@@ -21,16 +21,16 @@
 
 /**
  * SECTION:element-teletextdec
+ * @title: teletextdec
  *
  * Decode a stream of raw VBI packets containing teletext information to a RGBA
  * stream.
  *
- * <refsect2>
- * <title>Example launch line</title>
+ * ## Example launch line
  * |[
  * gst-launch-1.0 -v -m filesrc location=recording.mpeg ! tsdemux ! teletextdec ! videoconvert ! ximagesink
  * ]|
- * </refsect2>
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -231,10 +231,8 @@ gst_teletextdec_class_init (GstTeletextDecClass * klass)
       "Sebastian Pölsterl <sebp@k-d-w.org>, "
       "Andoni Morales Alastruey <ylatuya@gmail.com>");
 
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&src_template));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&sink_template));
+  gst_element_class_add_static_pad_template (gstelement_class, &src_template);
+  gst_element_class_add_static_pad_template (gstelement_class, &sink_template);
 }
 
 /* initialize the new element
@@ -659,12 +657,13 @@ static gboolean
 gst_teletextdec_negotiate_caps (GstTeletextDec * teletext, guint width,
     guint height)
 {
-  guint i, caps_size;
   gboolean rv = FALSE;
-
   /* get the peer's caps filtered by our own ones. */
   GstCaps *ourcaps = gst_pad_query_caps (teletext->srcpad, NULL);
   GstCaps *peercaps = gst_pad_peer_query_caps (teletext->srcpad, ourcaps);
+  GstStructure *caps_struct;
+  const gchar *caps_name, *caps_fmt;
+
   gst_caps_unref (ourcaps);
 
   if (gst_caps_is_empty (peercaps)) {
@@ -673,33 +672,26 @@ gst_teletextdec_negotiate_caps (GstTeletextDec * teletext, guint width,
 
   /* make them writable in case we need to fixate them (video/x-raw). */
   peercaps = gst_caps_make_writable (peercaps);
-  caps_size = gst_caps_get_size (peercaps);
+  caps_struct = gst_caps_get_structure (peercaps, 0);
+  caps_name = gst_structure_get_name (caps_struct);
+  caps_fmt = gst_structure_get_string (caps_struct, "format");
 
-  for (i = 0; i < caps_size; i++) {
-    GstStructure *caps_struct = gst_caps_get_structure (peercaps, i);
-    const gchar *caps_name = gst_structure_get_name (caps_struct);
-    const gchar *caps_fmt = gst_structure_get_string (caps_struct, "format");
-
-    if (g_strcmp0 (caps_name, "video/x-raw") == 0) {
-      teletext->width = width;
-      teletext->height = height;
-      teletext->export_func = gst_teletextdec_export_rgba_page;
-      gst_structure_set (caps_struct,
-          "width", G_TYPE_INT, width,
-          "height", G_TYPE_INT, height,
-          "framerate", GST_TYPE_FRACTION, 0, 1, NULL);
-      break;
-    } else if ((g_strcmp0 (caps_name, "text/x-raw") == 0) &&
-        (g_strcmp0 (caps_fmt, "utf-8") == 0)) {
-      teletext->export_func = gst_teletextdec_export_text_page;
-      break;
-    } else if ((g_strcmp0 (caps_name, "text/x-raw") == 0) &&
-        (g_strcmp0 (caps_fmt, "pango-markup") == 0)) {
-      teletext->export_func = gst_teletextdec_export_pango_page;
-      break;
-    } else {
-      goto beach;
-    }
+  if (!g_strcmp0 (caps_name, "video/x-raw")) {
+    teletext->width = width;
+    teletext->height = height;
+    teletext->export_func = gst_teletextdec_export_rgba_page;
+    gst_structure_set (caps_struct,
+        "width", G_TYPE_INT, width,
+        "height", G_TYPE_INT, height,
+        "framerate", GST_TYPE_FRACTION, 0, 1, NULL);
+  } else if (!g_strcmp0 (caps_name, "text/x-raw") &&
+      !g_strcmp0 (caps_fmt, "utf-8")) {
+    teletext->export_func = gst_teletextdec_export_text_page;
+  } else if (!g_strcmp0 (caps_name, "text/x-raw") &&
+      !g_strcmp0 (caps_fmt, "pango-markup")) {
+    teletext->export_func = gst_teletextdec_export_pango_page;
+  } else {
+    goto beach;
   }
 
   if (!gst_pad_push_event (teletext->srcpad, gst_event_new_caps (peercaps))) {
@@ -751,9 +743,7 @@ error:
   {
     if (ret != GST_FLOW_OK && ret != GST_FLOW_NOT_LINKED
         && ret != GST_FLOW_FLUSHING) {
-      GST_ELEMENT_ERROR (teletext, STREAM, FAILED,
-          ("Internal data stream error."), ("stream stopped, reason %s",
-              gst_flow_get_name (ret)));
+      GST_ELEMENT_FLOW_ERROR (teletext, ret);
       return GST_FLOW_ERROR;
     }
     return ret;
