@@ -24,6 +24,9 @@
 
 #include <gst/gst.h>
 #include <gst/video/video.h>
+
+#include <stdint.h>
+
 #ifdef G_OS_UNIX
 #include "linux/DeckLinkAPI.h"
 #endif
@@ -39,8 +42,8 @@
 #define FREE_COM_STRING(s) delete[] s;
 #define CONVERT_COM_STRING(s) BSTR _s = (BSTR)s; s = _com_util::ConvertBSTRToString(_s); ::SysFreeString(_s);
 #else
-#define COMSTR_T char*
-#define FREE_COM_STRING(s) free ((void *) s)
+#define COMSTR_T const char*
+#define FREE_COM_STRING(s)
 #define CONVERT_COM_STRING(s)
 #endif /* _MSC_VER */
 
@@ -110,6 +113,15 @@ typedef enum {
 #define GST_TYPE_DECKLINK_AUDIO_CONNECTION (gst_decklink_audio_connection_get_type ())
 GType gst_decklink_audio_connection_get_type (void);
 
+typedef enum {
+  GST_DECKLINK_AUDIO_CHANNELS_MAX = 0,
+  GST_DECKLINK_AUDIO_CHANNELS_2 = 2,
+  GST_DECKLINK_AUDIO_CHANNELS_8 = 8,
+  GST_DECKLINK_AUDIO_CHANNELS_16 = 16
+} GstDecklinkAudioChannelsEnum;
+#define GST_TYPE_DECKLINK_AUDIO_CHANNELS (gst_decklink_audio_channels_get_type ())
+GType gst_decklink_audio_channels_get_type (void);
+
 typedef enum {
   GST_DECKLINK_VIDEO_FORMAT_AUTO,
   GST_DECKLINK_VIDEO_FORMAT_8BIT_YUV, /* bmdFormat8BitYUV */
@@ -196,25 +208,21 @@ struct _GstDecklinkInput {
   IDeckLinkInput *input;
   IDeckLinkConfiguration *config;
   IDeckLinkAttributes *attributes;
-  GstClock *clock;
-  GstClockTime clock_start_time, clock_offset, clock_last_time, clock_epoch;
-  gboolean started, clock_restart;
 
   /* Everything below protected by mutex */
   GMutex lock;
 
   /* Set by the video source */
-  void (*got_video_frame) (GstElement *videosrc, IDeckLinkVideoInputFrame * frame, GstDecklinkModeEnum mode, GstClockTime capture_time, GstClockTime capture_duration, guint hours, guint minutes, guint seconds, guint frames, BMDTimecodeFlags bflags);
+  void (*got_video_frame) (GstElement *videosrc, IDeckLinkVideoInputFrame * frame, GstDecklinkModeEnum mode, GstClockTime capture_time, GstClockTime stream_time, GstClockTime stream_duration, IDeckLinkTimecode *dtc, gboolean no_signal);
   /* Configured mode or NULL */
   const GstDecklinkMode *mode;
   BMDPixelFormat format;
 
   /* Set by the audio source */
-  void (*got_audio_packet) (GstElement *videosrc, IDeckLinkAudioInputPacket * packet, GstClockTime capture_time, gboolean discont);
+  void (*got_audio_packet) (GstElement *videosrc, IDeckLinkAudioInputPacket * packet, GstClockTime capture_time, GstClockTime packet_time, gboolean no_signal);
 
   GstElement *audiosrc;
   gboolean audio_enabled;
-  gboolean audio_discont;
   GstElement *videosrc;
   gboolean video_enabled;
   void (*start_streams) (GstElement *videosrc);
